pub enum SamplingMode {
Greedy,
TopP {
p: Probability<f64>,
min_keep: NonZeroUsize,
},
TopK {
k: NonZeroUsize,
},
MinP {
p: Probability<f32>,
min_keep: NonZeroUsize,
},
TailFree {
z: Probability<f32>,
min_keep: NonZeroUsize,
},
LocallyTypical {
p: Probability<f32>,
min_keep: NonZeroUsize,
},
Mirostat {
tau: f32,
eta: f32,
max_keep: Option<NonZeroUsize>,
},
MirostatV2 {
tau: f32,
eta: f32,
max_keep: Option<NonZeroUsize>,
},
SplitP {
min_keep: NonZeroUsize,
max_keep: Option<NonZeroUsize>,
},
SplitL {
min_keep: NonZeroUsize,
max_keep: Option<NonZeroUsize>,
},
}Variants
Greedy
Greedy sampling. The most likely next token is always chosen. Not very useful unless you want to regurgitate the training data.
TopP
Top-p sampling. A token is chosen from the top tokens whose cumulative
probability is greater than or equal to p.
Fields
p: Probability<f64>Reasonable values are between 0.9 and 0.95. Higher means more diversity, but potentially less coherent.
min_keep: NonZeroUsizeMinimum number of candidates to keep per token.
TopK
A token is chosen from the top k tokens. This is not very good.
Fields
k: NonZeroUsizeThe top k tokens are kept. Reasonable values are between 30 and
40.
MinP
Min-p sampling. p sets the minimum probability to keep a token. Below
that the tail is cut off. p is scaled by the top token’s probability
to balance diversity and quality.
It is described in detail in the following pull request: https://github.com/ggerganov/llama.cpp/pull/3841
Fields
p: Probability<f32>The minimum probability to keep a token. This is scaled by the top token’s probability. Reasonable values are 0.05 to 0.3. Higher means less diversity.
min_keep: NonZeroUsizeTailFree
Tail free sampling.
“TFS first converts logits output by a model into probabilities using the softmax function before sorting them in descending order. It then calculates the first and second derivatives. As the tokens are discrete, this can be found with subtraction. The magnitude of each second derivative is then taken and normalized so that they sum to 1. Finally, a threshold z is used to determine what part of the cumulative distribution of the second derivative weights to define the “tail” of the distribution to be at.”
Fields
z: Probability<f32>Reasonable values are between 0.25 and 0.75. The higher, the more diverse the output, but also potentially less coherent.
min_keep: NonZeroUsizeMinimum number of candidates to keep per token.
LocallyTypical
Locally typical sampling.
“First, we compute the conditional entropy, which is an O(|V|)
operation. Second, we sort words by their absolute distance from H(pb(·|
Y <t = y<t)), which can be done in O(|V| log |V|) time with standard
sorting algorithms. Finally, we greedily take words from this list until
their cumulative probability exceeds the threshold p , which again
takes O(|V|) time. Thus, creating our altered distribution has time
complexity O(|V| log |V|).”
Fields
p: Probability<f32>Probability. Reasonable values are between 0.2 and 0.95. For story generation, lower is better. For summarization, higher is better.
min_keep: NonZeroUsizeMinimum number of candidates to keep per token.
Mirostat
Mirostat sampling.
“a neural text decoding algorithm that directly controls the perplexity of the generated text over a wide range of text length. Notably, for longer texts and certain ranges of input parameters, top-k and top-p sampling fall into boredom and confusion traps which cause low-quality texts; Mirostat avoids both traps.”
Fields
tau: f32Tau. Target entropy. A good value is 3.0 according to this paper: https://arxiv.org/pdf/2202.00666.pdf
llama.cpp uses a default of 5.0.
max_keep: Option<NonZeroUsize>Maximum number of candidates to keep. In the original paper and code
the default is 100 and the name is m.
MirostatV2
Mirostat V.2 sampling.
“Here we provide an alternate algorithm for perplexity control, Alg. 2, which does not depend on the distribution of the underlying LM. In this sense, Alg. 2 controls perplexity in more general sequential generative models than Alg. 1 where the underlying distribution may not be Zipfian. In our work, we choose Alg. 1 since it has only an additional constant time complexity compared to top-k sampling. Whereas Alg. 2 has additional time complexity that depends on target cross-entropy rate and vocabulary size, which may vary with different LMs.”
Note:
- The bit about time complexity is not relevant to this implementation since we truncate the candidates to a fixed size like v1.
Fields
tau: f32Tau. Target entropy. A good value is 3.0 according to the paper and HF’s experiments in https://arxiv.org/pdf/2202.00666.pdf
llama.cpp uses a default of 5.0.
max_keep: Option<NonZeroUsize>Maximum number of candidates to keep. Defaults to 100. The original implementation does not support this. If identical behavior is desired, set this to the vocabulary size.
SplitP
Split P sampling. This cuts the tail off where the difference between adjacent probabilities is greatest, where the slope is steepest.
Fields
min_keep: NonZeroUsizeMinimum number of candidates to keep.
max_keep: Option<NonZeroUsize>Maximum number of candidates to keep.
SplitL
Split L sampling. This cuts the tail off where the difference between adjacent logits is greatest, where the slope is steepest.
Fields
min_keep: NonZeroUsizeMinimum number of candidates to keep.
max_keep: Option<NonZeroUsize>Maximum number of candidates to keep.
Implementations
Sourceimpl SamplingMode
impl SamplingMode
pub const ALL: [Self; 10]
Sourcepub const fn help(&self) -> &'static str
pub const fn help(&self) -> &'static str
A help message for the sampling mode (but not it’s parameters)
Sourcepub const fn locally_typical() -> Self
pub const fn locally_typical() -> Self
Default locally typical sampling: p = 0.5 with no minimum keep.
Sourcepub const fn mirostat() -> Self
pub const fn mirostat() -> Self
Default mirostat sampling: tau = 3.0, eta = 0.1, max_keep = 100.
Sourcepub const fn mirostat_v2() -> Self
pub const fn mirostat_v2() -> Self
Default mirostat v2 sampling: tau = 3.0, eta = 0.1, max_keep = 100.
Sourcepub fn draw_inner(&mut self, ui: &mut Ui) -> Response
pub fn draw_inner(&mut self, ui: &mut Ui) -> Response
Draw egui::Ui, but without the outer collapsible header.